// $Id: pac_output.cc,v 1.3 2005/09/30 20:48:04 vern Exp $

#include <string.h>
#include <errno.h>

#include "pac_utils.h"
#include "pac_output.h"

OutputException::OutputException(const char* arg_msg)
	{
	msg = copy_string(arg_msg);
	}

OutputException::~OutputException()
	{
	delete [] msg;
	}

OutputException& OutputException::operator=(const OutputException& oe)
	{
	if ( msg)
		delete [] msg;

	msg = copy_string(oe.msg);

	return *this;
	}

Output::Output(const char* filename)
	{
	fp = fopen(filename, "w");
	if ( ! fp )
		{
		// XXX  Can't use fmt() here because filename is already
		// XXX  resulting from fmt() when called from compile(),
		// XXX  and we only have 1 static buffer.
		//
		char buf[1024];
		snprintf(buf, sizeof(buf), "cannot open '%s' for writing, %s", filename, strerror(errno));
		throw OutputException(buf);
		}
	indent = 0;
	}

Output::~Output()
	{
	if ( fp )
		fclose(fp);
	}

int Output::print(const char* fmt, va_list ap)
	{
	int r = vfprintf(fp, fmt, ap);
	if ( r == -1 )
		throw OutputException(strerror(errno));
	return r;
	}

int Output::print(const char* fmt, ...)
	{
	va_list ap;
	va_start(ap, fmt);
	return print(fmt, ap);
	}

int Output::println(const char* fmt, ...)
	{
	for ( int i = 0; i < indent; ++i )
		fprintf(fp, "\t");

	int r;
	va_list ap;
	va_start(ap, fmt);
	r = print(fmt, ap);

	fprintf(fp, "\n");
	return r;
	}
